require 'open-uri'

module Integration
  module ParserHelpers
    def get_ubuntu_index(dist, url)
      url = case (dist.downcase)
            when 'precise'
              URI.join(url, "dists/#{dist.downcase}/main/binary-amd64/Packages.gz")
            else
              raise "Unknown dist: #{dist}"
            end
      Zlib::GzipReader.new(open(url))
    end

    def dpkg_deb(field, deb_path)
      field = capitalize_deb_field(field)
      `dpkg-deb --showformat '${#{field}}' --show #{deb_path}`
    end

    def is_relationship?(fieldname)
      PackagecloudDebtools::Parser::RELATIONSHIP_FIELDS.include?(fieldname)
    end

    def get_package_file(filename, url, cachedir)
      cache_file_path = File.join(cachedir, filename)
      if !::File.exists?(cache_file_path)
        open(cache_file_path, 'wb') do |file|
          file << open(url).read
        end
      end
      cache_file_path
    end

    def sum_from_file(sum_type, file_path)
      case(sum_type.downcase)
      when 'sha1sum'
        Digest::SHA1.file(file_path).hexdigest
      when 'sha256sum'
        Digest::SHA256.file(file_path).hexdigest
      when 'sha512sum'
        Digest::SHA512.file(file_path).hexdigest
      when 'md5sum'
        Digest::MD5.file(file_path).hexdigest
      else
        raise "unknown sum_type: #{sum_type}"
      end
    end

    def capitalize_deb_field(field)
      case(field.downcase)
      when 'sha1sum', 'sha1'
        'SHA1'
      when 'sha256sum', 'sha256'
        'SHA256'
      when 'sha512sum', 'sha512'
        'SHA512'
      when 'md5sum', 'md5'
        'MD5sum'
      when 'description-md5'
        'Description-md5'
      else
        field.split("-").map(&:capitalize).join("-")
      end
    end

    def get_list_of_pkgs(index_file_io, max_size)
      buffer = ""
      size = nil
      pkgs = []
      package = nil
      filename = nil

      index_file_io.each_line do |line|
        buffer = buffer + line

        if line =~ /^Size\:\s+(.*)/
          size = $1.to_i
        elsif line =~ /^Package\:\s+(.*)/
          package = $1
        elsif line =~ /^Filename\:\s+(.*)/
          filename = $1
        end

        if line == "\n" && size <= max_size
          pkgs << { size: size,
            package: package,
            filename:  filename,
            stanza: buffer }
          buffer = ""
        elsif line == "\n"
          buffer = ""
        end
      end

      pkgs = pkgs.shuffle.shuffle.shuffle
      index_file_io.rewind
      pkgs
    end
  end
end
